################################################################################
### Bar.pm (Yggdrasil::Progress)
### Percent-bar progress indicator
###
### Copyright (C) 2003 Morten Wulff <wulff@ratatosk.net>
###
### This program is free software; you can redistribute it and/or
### modify it under the terms of the GNU General Public License
### as published by the Free Software Foundation; either version 2
### of the License, or (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
################################################################################

### REVISIONS
###	1.0.0 : 2003-04-24 : Morten Wulff <wulff@ratatosk.net>

package Yggdrasil::Progress::Bar;

use strict;
use warnings;

$|++;

use Carp;
use Yggdrasil::Progress;

@Yggdrasil::Progress::Bar::ISA = qw(Yggdrasil::Progress);

################################################################################
### INIT
################################################################################

my $Scale = <<SCALE;
0%  10%  20%  30%  40%  50%  60%  70%  80%  90%  100%
|----|----|----|----|----|----|----|----|----|----|
SCALE

my %Fields = (
	marker   => '>', ### used when 'in range'
	clip	 => '<', ### used when 'out of range' (e.g. counter > 100)
	step     => 2,   ### 100% scale / 50 characters wide
	elements => 0,
	scale    => $Scale
);

################################################################################
### PUBLIC METHODS
################################################################################

################################################################################
sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;

	my $self = bless($class->SUPER::new(), $class);

	foreach my $element (keys %Fields) {
		$self->{_permitted}->{$element} = $Fields{$element};
	}

	@{$self}{keys %Fields} = values %Fields;

	$self->{elements} = shift;

	### check for presence of required parameter
	unless ( $self->{elements} && $self->{elements} > 0 ) {
		croak 'You need to specify a number of elements greater than zero';
	}

	### initialize counters
	$self->{item}          = 0;
	$self->{scale_printed} = 0;
	$self->{clip_printed}  = 0;
	$self->{clip_counter}  = 1;

	return $self;
}

################################################################################
sub update {
	my $self = shift;

	unless ($self->{scale_printed}) {
		print $self->{scale};
		print $self->{marker};
		$self->{scale_printed}++;
	}

	$self->{item}++;
	my $percentile = $self->{item} / $self->{elements} * 100;

	while ($self->{counter} < $percentile) {
		$self->{counter} += $self->{step};
		print "\b $self->{marker}" if $self->{counter} <= 100;;
	}

	if ($self->{counter} > 100) {
		unless ($self->{clip_printed}) {
			print "\b $self->{clip}      ";
			$self->{clip_printed}++;
		}
		printf "\b\b\b\b\b\b%-6u", $self->{clip_counter};
		$self->{clip_counter}++;
	}
}

1;
